class WorkflowNotFound(Exception):
    pass


class SchemaNotFound(Exception):
    pass


class SchemaNotValid(Exception):
    pass


class WorkflowSyntaxError(Exception):
    pass


class UserNotFound(Exception):
    pass
