<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <title>%% error.code %% %% error.name %%</title>
  </head>
  <body>
    <h1>%% error.code %% %% error.name %%</h1>
    <p>%% error.description %%</p>
  </body>
</html>